/****************************************************************************************** 
|
|   Simian 
|   Copyright (c) 2009-2020+ Mountain Technology Group, llc.  / Simian
|   All Rights Reserved. www.gosimian.com
|  
|   It is unlawful to reproduce, copy or otherwise reuse this software
|   without express written permission of Simian.  
|
******************************************************************************************/ 

// Simian HTML5 Video Player
(function($) {
	$.fn.simianVideoPlayer = function(options) {
		
		// Player Options
		var defaults = {
			theme: 'admin',
			quePoints: false,
			trackAnalytics: false,
			trackViewCallback:false,
			trackPartialCallback:false,
			endVideoCallback:false,
			airplay: false,
			trimPlayback: false,
			trimScrub: false,
			trimStart: 0,
			trimStop: 0,
			trimUpdateCallback: false,
			videoDuration: 0,
			drawingTools: false,
			closedCaptions: false,
			autoPlay: false,
			playerWidth: '100%',
			playerHeight: 'auto',
			fullScreen: true,
			fullScreenTop: false,
			engageControls: false, 
			engageData: false,
			moodReactions: false,
			loopPlayback: false,
			frameControls: false,
			timeFormatControl: false,
			frameRate: 0,
			timeStampComments: false,
			gridOverlay: false,
			playButtonOverlay: false,
			timeCodeOffset: 0,
			initializeCallback: function () { },
		};
		options = $.extend(defaults, options);
		
		function supports_local_storage() {
			try {
				return 'localStorage' in window && window.localStorage !== null;
			} catch(e){
				return false;
			}
		}
		
		var volume = (supports_local_storage) ? localStorage.getItem('simianvolume') : 1;
		if(!volume) {
			volume = 1;
		}

		var timeDisplay = (supports_local_storage) ? localStorage.getItem('simiantimedisplay') : 1;
		if(timeDisplay && options.timeFormatControl) {
			// Do nothing		
		}
		else
			timeDisplay = 'standard';

		var videoFile = '';
		var autoplay = options.autoPlay;

		var updateTimer = '';
		var endVideoCallBack = false;

		// Analytics Variables
		var playerCurrentTime = 0;
		var analyticsStartTime = 0;
		var analyticsStopTime = 0;
	  	var firstUnloadCall = true;
	  	var reversePlay = false;
			
		// Trim Variables
		var trimStart = options.trimStart;
		var trimStop = options.trimStop;
		var trimPlaybackStopped = false;
		
		// Range Variables
		var rangeStart = 0;
		var rangeStop = 0;
				
		var loopMode = false;
		var fullscreenMode = false; // The media is in fullscreen mode
		var simPlayer = null;
	
		var simianPlayer = function() {
			// set the main video object, will contain the needed DOM nodes and others
			simPlayer = {
				$self: $(this)
			};

			var loadedMetadata; // Is the metadata loaded
			var seeking; // The user is seeking the media
			var wasPlaying; // Media was playing when the seeking started
			var videoDuration = options.videoDuration;
			var is_touch_device = 'ontouchstart' in document.documentElement;

			var text = {
				play: 'Play',
				playTitle: 'Start the playback',
				pause: 'Pause',
				pauseTitle: 'Pause the playback',
				mute: 'Mute',
				unmute: 'Unmute',
				fullscreen: 'Fullscreen',
				fullscreenTitle: 'Toggle fullscreen mode',
				volumeTitle: 'Volume control',
				seekTitle: 'Video seek control',
			};

			// disable native autoplay
			simPlayer.$self.prop('autoplay', false);

			// main wrapper element
			var $playerWrapper = $('<div class="simian-player" role="application" data="simianVideoPlayer" style="position:relative;"></div>');

			if(options.playButtonOverlay)
				$playerWrapper.append('<div class="videoPlayOverlay"></div>');

			// Ability to tab to video player
			simPlayer.$self.attr('tabindex', '0');

			var playerUI  = '';
			
			// Will have to change this for use outside of Simian			
			playerUI += '<link rel="stylesheet" type="text/css" href="' + base_url + 'media/common/vplayer/skin/simian-' + options.theme + '.css"/>';

			playerUI +=	'	<div class="sp-Info"><img src="' + base_url + 'media/common/images/admin-logo-white.png" style="width:25px;vertical-align:middle;margin-right:10px;"> Simian HTML5 Player v4.21<br/><br/>www.gosimian.com</div>';
			playerUI +=	'	<div class="sp-Error"></div>';

			if(options.fullScreenTop)
				playerUI +=	'	<div class="sp-FullScreen"></div>';

			if(options.airplay && window.WebKitPlaybackTargetAvailabilityEvent)
			{				
				playerUI +=	'	<div class="sp-Airplay"></div>';
			}
			else	
				options.airplay = false;	
				
			playerUI += '<div class="sp-Scrub" style="width:' + options.playerWidth + ';height:20px;">';
			playerUI +=	'		<div class="sp-Scrub_buffer"></div>';
			playerUI +=	'		<div class="sp-Scrub-slider"></div>';
			
			if(options.trimPlayback)
			{
				playerUI +=	'<div class="trimStartMask"></div><div class="trimStart"></div>';
				playerUI +=	'<div class="trimStop"></div><div class="trimStopMask"></div>';
			}
			
			playerUI +=	'	</div>'; // End sp-scrub

			if(options.drawingTools)
			{
				playerUI +=	'<div id="dynamicLoad" class="dcontrols"><div id="cmtDisplay"></div></div>';
			}

			playerUI +=	'	<div class="sp-Ctrls" style="width:' + options.playerWidth + ';float:left;margin:0px;">';
			playerUI +=	'	<div class="sp-PlayBtn"></div>';
			
			if(options.timeFormatControl)
			{
				playerUI +=	'	<div class="sp-TimeFormat">';
			}
			
			playerUI +=	'	<div class="sp-CurrentTime">00:00</div>';

			if(options.timeFormatControl)
			{
				playerUI +=	'	<div class="sp-TimeDivider">:</div><div class="sp-TotalTime">----</div>';
				playerUI +=	'	<div class="sp-TimeFormatSelect"><a href="#" value="standard">Standard Format</a><br/><a href="#" value="timecode">Time Code</a><br/><a href="#" value="framecount">Frame Count</a></div>';
				playerUI +=	'	</div>';
			}
			
			if(options.gridOverlay)
			{
				playerUI +=	'	<div class="sp-GridMenu"><span>ASPECT MASK : OFF</span>';
				playerUI += ' 	<div class="sp-GridMenuSelect">'
				playerUI +=	'	<a href="#" value="off">TURN OFF MASK</a><br/><a href="#" value="16:9">16:9 High Definition</a><br/><a href="#" value="4:3">4:3 Standard Definition</a><br/><a href="#" value="21:9">21:9 Cinemascope</a><br/><a href="#" value="1:1">1:1 Square</a>';
				playerUI +=	'	</div></div>';
			}

			playerUI +=	'	<div class="sp-VolumeBtn">';
			playerUI +=	'		<div class="sp-player-volume-adjust">';
			playerUI +=	'			<div class="sp-volume-slider"></div>';
			playerUI +=	'		</div>';
			playerUI +=	'	</div>';

			if(options.fullScreen)
				playerUI +=	'	<div class="sp-FullScreen"></div>';

			if(options.moodReactions)
				playerUI +=	'	<div class="sp-emoji-btn"></div>';

			if(options.loopPlayback)
				playerUI +=	'	<div class="sp-LoopPlayback"></div>';
				
			if(options.frameControls)
				playerUI +=	'	<div class="sp-FrameForward" title="Frame Forward"></div><div class="sp-FrameBack" title="Frame Back"></div>';

			if(!options.timeFormatControl)	
				playerUI +=	'	<div class="sp-TotalTime">----</div>';
			
			playerUI +=	'</div>';
			
			// append the wrapper
			simPlayer.$self.after($playerWrapper);
			
			// iOS Specific	
			$playerWrapper[0].appendChild( simPlayer.$self[0].cloneNode(true) );

			simPlayer.$self.remove();
			simPlayer.$self = $playerWrapper.find('video');

			if(options.theme == 'reel')
				simPlayer.$self.prop('preload','metadata');

			// Append no download to controls
			simPlayer.$self.prop('controlsList','nodownload');

			// append the controls and loading mask
			simPlayer.$self.after(playerUI).after('<div class="loading-media"></div>');

			// Set up DOM nodes
			simPlayer.$container = simPlayer.$self.parent('.simian-player');

			simPlayer.$controls = $('.sp-Ctrls', simPlayer.$container);
			simPlayer.$playBtn = $('.sp-PlayBtn', simPlayer.$container);
			simPlayer.$scrubBar = $('.sp-Scrub', simPlayer.$container);
			simPlayer.$seek = $('.sp-Scrub-slider', simPlayer.$container);
			simPlayer.$buffer = $('.sp-Scrub_buffer', simPlayer.$container);
			simPlayer.$timer = $('.sp-CurrentTime', simPlayer.$container);
			simPlayer.$totalTime = $('.sp-TotalTime', simPlayer.$container);
			simPlayer.$volume = $('.sp-player-volume-adjust', simPlayer.$container);
			simPlayer.$volumeBtn = $('.sp-VolumeBtn', simPlayer.$container);
			simPlayer.$volumeControl = $('.sp-volume-slider', simPlayer.$container);
			simPlayer.$playerInfo = $('.sp-Info', simPlayer.$container);
			simPlayer.$errorInfo = $('.sp-Error', simPlayer.$container);

			if(options.timeFormatControl)
			{
				simPlayer.$timeFormat = $('.sp-TimeFormat', simPlayer.$container);
				simPlayer.$timeFormatSelect = $('.sp-TimeFormatSelect', simPlayer.$container);
				simPlayer.$timeFormatOption = $('.sp-TimeFormatSelect a', simPlayer.$container);
			}
			
			if(options.gridOverlay)
			{
				simPlayer.$gridMenu = $('.sp-GridMenu', simPlayer.$container);
				simPlayer.$gridMenuSelect = $('.sp-GridMenuSelect', simPlayer.$container);
				simPlayer.$gridMenuOption = $('.sp-GridMenuSelect a', simPlayer.$container);
			}
			
			if(options.loopPlayback)
				simPlayer.$loopPlayback = $('.sp-LoopPlayback', simPlayer.container);	

			if(options.frameControls)
			{
				simPlayer.$frameForward = $('.sp-FrameForward', simPlayer.container);	
				simPlayer.$frameBack = $('.sp-FrameBack', simPlayer.container);	
			}
		
			if(options.airplay)
				simPlayer.$airPlayBtn = $('.sp-Airplay', simPlayer.$container);
			
			// trimPlayback controls
			if(options.trimPlayback)
			{
				simPlayer.$trimHandleStart = $('.trimStart', simPlayer.$container);
				simPlayer.$trimHandleStop = $('.trimStop', simPlayer.$container);
				simPlayer.$trimMaskStart = $('.trimStartMask', simPlayer.$container);
				simPlayer.$trimMaskStop = $('.trimStopMask', simPlayer.$container);
			}
			
			// Engagement Graph controls
			if(options.engageControls)
			{
				simPlayer.$engageLabel = $('.graphScrubLabel');
				simPlayer.$engageOverlay = $('#overlayGraph');
			}
					
			simPlayer.$fullscreenBtn = $('.sp-FullScreen', simPlayer.$container);
									
			// Time Formatting
			var timeFormat = function(sec) {
				if(timeDisplay == 'standard')
				{
					if(sec > 3600)
					{
						var h = Math.floor(sec/3600)<10?"0" + Math.floor(sec/3600):Math.floor(sec/3600);
						sec %= 3600;
						var m = Math.floor(sec/60)<10?"0" + Math.floor(sec/60):Math.floor(sec/60);
						var s = Math.floor(sec-(m*60))<10?"0" + Math.floor(sec-(m*60)):Math.floor(sec-(m*60));
						return h + ":" + m + ":" + s;
					}
					else {
						var m = Math.floor(sec/60)<10?"0" + Math.floor(sec/60):Math.floor(sec/60);
						var s = Math.floor(sec-(m*60))<10?"0" + Math.floor(sec-(m*60)):Math.floor(sec-(m*60));
						return m + ":" + s;
					}
				}
				else if(timeDisplay == 'timecode')
				{
					sec += parseFloat(options.timeCodeOffset);
				    var pad = function(num, size) { return ('000' + num).slice(size * -1); },
				    frames = Math.floor((sec % 1) * options.frameRate),
				    time =  sec, 
				    hours = Math.floor(time / 60 / 60),
				    minutes = Math.floor(time / 60) % 60,
				    seconds = Math.floor(time - minutes * 60);
					frames -= 1;
					if(frames < 0)
						frames = 0;
					
					if(options.frameRate > 100)
						return pad(hours, 2) + ':' + pad(minutes, 2) + ':' + pad(seconds, 2) + ':' + pad(frames,3);
					else
						return pad(hours, 2) + ':' + pad(minutes, 2) + ':' + pad(seconds, 2) + ':' + pad(frames,2);
				}
				else if(timeDisplay == 'framecount')
				{					
					if(options.frameRate > 0)
					{
						sec += parseFloat(options.timeCodeOffset);
					
						var totalDuration = simPlayer.$self.prop('duration');
						var totalFrames = (totalDuration * options.frameRate);	
						var frameCount = Math.floor(((sec*totalFrames)/totalDuration));				
						frameCount -= 1;
						
						if(frameCount < 0)
							frameCount = 0;	

						return frameCount;
					}
					else
					{	// Just in case the frameRate is empty
						var m = Math.floor(sec/60)<10?"0" + Math.floor(sec/60):Math.floor(sec/60);
						var s = Math.floor(sec-(m*60))<10?"0" + Math.floor(sec-(m*60)):Math.floor(sec-(m*60));
						return m + ":" + s;
					}
				}
			};
			
			// For External Access
			simPlayer.$formatTheTime = timeFormat;

			// Ready to go
			var initVideo = function() {
				
				if(options.trimScrub && (trimStart != '' || trimStop != ''))
				{
					var trimDuration = (trimStop - trimStart);
					simPlayer.$totalTime.html(timeFormat(trimDuration));
				}
				else
					simPlayer.$totalTime.html(timeFormat(simPlayer.$self.prop('duration')));
	
				if(videoDuration == 0)
					videoDuration = simPlayer.$self.prop('duration');
					
				// Call any initilization that needs to happen after player loaded					
				options.initializeCallback.call(this);	
			}

			// Play / Pause
			var playMedia = function() {

				reversePlay = false;
				
				if(!simPlayer.$self.prop('paused')) 
				{
					// Pause
					simPlayer.$self.trigger('pause');
				} 
				else 
				{
					trimPlaybackStopped = false;
					
					if(options.playButtonOverlay)
						$('.videoPlayOverlay').remove();

					// Play
					simPlayer.$self.trigger('play');
				}
				
				simPlayer.$self.focus();
				return false;
			};

			// Native events
			var startPlayback = function() {
				// For good measure
				videoDuration = simPlayer.$self.prop('duration');
				
				simPlayer.$playBtn.addClass('sp-Pause');

				// Add more updates for a smoother scrub bar
				if(updateTimer == '')
				{
					updateTimer = setInterval(function () {
						showProgress();	
					}, 100);
				}

				if(options.playButtonOverlay)
					$('.videoPlayOverlay').remove();

				// if the metadata is not loaded yet, add the loading class
				if(!loadedMetadata) $playerWrapper.addClass('show-loading');
				
				// if drawing tools are on - clear the current annoation image
				if(options.drawingTools)
				{
					if(typeof(exitDraw) == 'function')
						exitDraw();
					if(typeof(simHideComment) == 'function')
						simHideComment();
				}
				
				if(options.trackAnalytics)
				{
					if(!seeking) 
					{
						if(simPlayer.$self.prop('currentTime') != undefined)
							analyticsStartTime = simPlayer.$self.prop('currentTime');	
					}
				}
				
				if(options.trimPlayback && (playerCurrentTime < trimStart || playerCurrentTime > trimStop) && (trimStart != 0 && trimStop != 0))
				{			
					simPlayer.$self.prop('currentTime', trimStart);
					analyticsStartTime = trimStart;
				}	
				
				if((playerCurrentTime < rangeStart || playerCurrentTime > rangeStop) && (rangeStart != 0 && rangeStop != 0))
				{
					simPlayer.$self.prop('currentTime', rangeStart);
				}
				
			};

			var stopPlayback = function() { // Pause button
				
				simPlayer.$playBtn.removeClass('sp-Pause');	

				clearInterval(updateTimer);
				updateTimer = '';

				if(options.trackAnalytics)
				{
					if(!seeking && !simPlayer.$self.prop('paused')) 
					{
						if(!trimPlaybackStopped)
						{
							analyticsStopTime = simPlayer.$self.prop('currentTime');
							options.trackPartialCallback(videoDuration, analyticsStopTime, analyticsStartTime, analyticsStopTime);
						}
					}
				}
			};

			var endPlayback = function() {
				
				var hitEndOfPlayback = false;
				
				if(options.trackAnalytics)
				{
					analyticsStopTime = simPlayer.$self.prop('currentTime');
					var elapsed = analyticsStopTime - analyticsStartTime;
					elapsed = (elapsed / videoDuration) * 100;

					if(elapsed > 99)
					{
						options.trackViewCallback('full', 100,analyticsStartTime, analyticsStopTime);	
					}
					else
						options.trackViewCallback('partial', elapsed,analyticsStartTime, analyticsStopTime);	
				}
				
				if(loopMode)
				{
					if(options.trimPlayback && trimStart != '' && trimStart > 0)
					{
						simPlayer.$self.prop('currentTime', trimStart);
						trimPlaybackStopped = false;
					}
					else	
						simPlayer.$self.prop('currentTime', 0);

					simPlayer.$self.trigger('play');
					return;
				}
				
				simPlayer.$playBtn.removeClass('sp-Pause');

				if(options.endVideoCallback)
				{
					endVideoCallBack = true;
					options.endVideoCallback();
				}
			};

			var showBuffer = function(e) {
				var vidduration = parseFloat(simPlayer.$self.prop('duration'), 10);
				var buffered = simPlayer.$self.prop('buffered');
				if(buffered && buffered.length) {
					var buffer = parseFloat(buffered.end(0) - buffered.start(0), 10);
					var bufferWidth = (simPlayer.$scrubBar.width() * (buffer/vidduration));
					simPlayer.$buffer.css('width',bufferWidth+'px');	
				}
			};

			var showProgress = function() {

				// IF we aren't doing anything don't update progress
				/*if(!seeking && simPlayer.$self.prop('paused'))
				{
					return;
				}*/	

				try
				{				
					playerCurrentTime = simPlayer.$self.prop('currentTime');
	
					if(options.trimScrub && (trimStart != '' || trimStop != ''))
					{
						if(playerCurrentTime > 0)
							playerUpdateTime = playerCurrentTime - trimStart;		
						else	
							playerUpdateTime = 0;	
					}
					else	
						playerUpdateTime = playerCurrentTime;
	
					simPlayer.$timer.text(timeFormat(playerUpdateTime));
					if($('#sp-timeStampUpdate').length > 0)
					{
						if(rangeStop > 0 && rangeStop > rangeStart)
						{
							$('#sp-timeStampUpdate').text(timeFormat(rangeStart) + ' - ' + timeFormat(rangeStop));
						}
						else
							$('#sp-timeStampUpdate').text(timeFormat(playerUpdateTime));
					}
	
					// If the user is not manualy seeking
					if(!seeking) {
						simPlayer.$seek.slider('value', playerUpdateTime);
					}
					
					// trimPlayback Options
					if(options.trimPlayback && playerCurrentTime >= trimStop && trimStop != 0 && playerCurrentTime != 0)
			    	{
						if(!trimPlaybackStopped && playerCurrentTime != 0)
						{
							trimPlaybackStopped = true;
							simPlayer.$self.trigger('pause');

							if(playerCurrentTime != 0 && trimStop != '')
							{
								endPlayback();
								return;
							}
						}
					}

					if(playerCurrentTime >= rangeStop && rangeStop != 0)
			    	{
				    	if(!seeking && !simPlayer.$self.prop('paused'))
				    	{
							simPlayer.$self.trigger('pause');
							endPlayback();
						}
					}
					
					// engageControl Options
					if(options.engageControls && options.engageData)
					{
				    	cView = options.engageData[Math.trunc(playerCurrentTime)];
				    	simPlayer.$engageLabel.css('display','block');
				    	simPlayer.$engageLabel.css('left',$('.sp-Scrub-slider .ui-slider-range').width());
				    	simPlayer.$engageOverlay.css('width',$('.sp-Scrub-slider .ui-slider-range').width());
				    	var prt = Math.ceil(playerCurrentTime);
						
				    	if(cView == 1)
				    	{
					    	simPlayer.$engageLabel.html( cView + ' View');
				    	}
				    	else
					    {
							simPlayer.$engageLabel.html( cView + ' Views');
				    	}
					}
				}
				catch(err)
				{
					// Catch any js errors
				}
			};

			var startSeek = function(e, ui) {
						
				// if drawing tools are on - clear the current annoation image
				if(options.drawingTools)
				{
					if(typeof(exitDraw) == 'function')
						exitDraw();
					if(typeof(simHideComment) == 'function')
						simHideComment();
				}
				
				if(!simPlayer.$self.prop('paused')) {
					wasPlaying = true;
				}
				simPlayer.$self.trigger('pause');
				seeking = true;

				var seekTime = ui.value;
				
				if(options.trimScrub && (trimStart != '' || trimStop != ''))
					seekTime = (parseFloat(seekTime) + parseFloat(trimStart));

				// engageControl Options
				if(options.engageControls && options.engageData)
				{
			    	cView = options.engageData[Math.trunc(playerCurrentTime)];
			    	simPlayer.$engageLabel.css('display','block');
			    	simPlayer.$engageLabel.css('left',$('.sp-Scrub-slider .ui-slider-range').width());
			    	simPlayer.$engageOverlay.css('width',$('.sp-Scrub-slider .ui-slider-range').width());
			    	var prt = Math.ceil(playerCurrentTime);
					
			    	if(cView == 1)
			    	{
				    	simPlayer.$engageLabel.html( cView + ' View');
			    	}
			    	else
				    {
						simPlayer.$engageLabel.html( cView + ' Views');
			    	}
				}


				simPlayer.$self.prop('currentTime', seekTime);
			};

			var endSeek = function(e, ui) {
				if(wasPlaying) {
					simPlayer.$self.trigger('play');
					wasPlaying = false;
				}
				seeking = false;				
			};

			var initSeek = function() {
				// get existing classes
				var seekClass = simPlayer.$seek.attr('class');

				// create the new markup
				var	divSeek = '<div class="' + seekClass + '"></div>';
				simPlayer.$seek.after(divSeek).remove();

				// get the newly created DOM node
				simPlayer.$seek = $('.' + seekClass, simPlayer.$container);

				// initialize slider 
				simPlayer.$seek.slider();
			};

			// Update 
			var updateSeek = function() {
				
				if(rangeStart != 0 && rangeStop != 0)
					return;
				
				// Get the duration of the media
				var duration = simPlayer.$self.prop('duration');

				if(options.trimScrub && (trimStart != '' || trimStop != ''))
				{
					duration = trimStop - trimStart;
				}

				var stepInc = 0.001; 
				if(options.frameRate > 0)
				{
					stepInc = (1 / options.frameRate);
					stepInc.toFixed(6);
				}

				// Initialize the slider
				var sliderOptions = {
					value: 0,
					range: 'min',
					step: stepInc,
					orientation: 'horizontal',
					min: 0,
					max: duration,
					slide: startSeek,
					stop: endSeek,
					label: 'Video seek control'
				};
				simPlayer.$seek.slider('option', sliderOptions);

				// remove the loading class
				$playerWrapper.removeClass('show-loading');
			};

			var setVolume = function(e, ui) {
				simPlayer.$self.prop('volume', ui.value);
			}

			var initVolume = function() {
				var volOptions = {
					value: 1,
					range: 'min',
					step: 0.05,
					orientation: 'vertical',
					min: 0,
					max: 1,
					stop: setVolume,
				};
				simPlayer.$volumeControl.slider();
				simPlayer.$volumeControl.slider('option', volOptions);
			};

			var resizeFullscreenVideo = function() {
				simPlayer.$self.attr({
					'width': $(window).width(),
					'height': $(window).height()
				});
			};

			var exitFullscreen = function() {
			    if (!document.fullscreenElement && !document.webkitIsFullScreen && !document.mozFullScreen && !document.msFullscreenElement) {
					fullscreenMode = false;
					
					// Projects full screen adjustment for safari
					$('#simianPlayer').removeClass('isProjFullscreen');
					$('#simianPlayer').removeAttr("style");					
			    }
			}

			var goFullscreen = function() {
				if(fullscreenMode) {

					// Projects full screen adjustment for safari
					$('#simianPlayer').removeClass('isProjFullscreen');

					// exit fullscreen
					if(simPlayer.$self[0].cancelFullScreen) {
						simPlayer.$self[0].cancelFullScreen();
					} else if(simPlayer.$self[0].webkitCancelFullScreen) {
						simPlayer.$self[0].webkitCancelFullScreen();
					} else if(simPlayer.$self[0].mozCancelFullScreen) {
						simPlayer.$self[0].mozCancelFullScreen();
					} else {
						alert('Your browser does not support fullscreen');
					}

					fullscreenMode = false;

				} else {
									
					// Projects full screen adjustment for safari
					$('#simianPlayer').addClass('isProjFullscreen');
									
					if(simPlayer.$self[0].requestFullscreen) {
						simPlayer.$self[0].requestFullscreen();
					} else if(simPlayer.$self[0].webkitRequestFullscreen) {
						simPlayer.$self[0].webkitRequestFullscreen();
					} else if(simPlayer.$self[0].mozRequestFullScreen) {
						simPlayer.$self[0].mozRequestFullScreen();
					} else {
						alert('Your browser does not support fullscreen');
					}

					fullscreenMode = true;
				}
			};
			
			var trimUpdate = function() {
				if(options.trimUpdateCallback)
					options.trimUpdateCallback(trimStart,trimStop);
			}
			
			// Trim Playback Feature
			var initTrimPlayback = function() {
								
		  	    var startLeft = ((simPlayer.$controls.width()*trimStart)/videoDuration);
		  	    var stopLeft = ((simPlayer.$controls.width()*trimStop)/videoDuration) - 25;
		  	    
		  	    if(startLeft <= 0)
		  	    	startLeft = 0;
		  	    if(stopLeft <= 0)
		  	    	stopLeft = 'calc(100% - 25px)';
		  	    
				simPlayer.$trimHandleStart.css('left',startLeft);
				simPlayer.$trimHandleStop.css('left',stopLeft);
		
				simPlayer.$trimMaskStart.css('width',startLeft);	
				simPlayer.$trimMaskStop.css('width',simPlayer.$scrubBar.width() - stopLeft);	
								
				simPlayer.$trimMaskStop.click(function(e) { e.stopPropagation(); });
				simPlayer.$trimMaskStart.click(function(e) { e.stopPropagation(); });
			
				simPlayer.$trimHandleStart.draggable({containment: 'parent',
					drag: function(e, ui) {
						e.stopPropagation();
			
						if(ui.position.left >= simPlayer.$trimHandleStop.position().left)
							ui.position.left = simPlayer.$trimHandleStop.position().left;
			
						simPlayer.$trimMaskStart.css('width',$(this).position().left);	
					},
					stop: function (e, ui) {
						
						if ($(e.target).attr('class') === ".sp-Scrub") {
						e.stopPropagation();
			            	return false;
			        	} 

						if(ui.position.left >= simPlayer.$trimHandleStop.position().left)
							ui.position.left = simPlayer.$trimHandleStop.position().left;
			
						simPlayer.$trimMaskStart.css('width',$(this).position().left);

						var parentOffset = $(this).parent().offset(); 
					  	var relX = e.pageX - parentOffset.left - 12.5;
					  	
					  	if(videoDuration * (relX/$(this).parent().width()) < 0)
					  		trimStart = 0;
					  	else
					  	    trimStart = (videoDuration * (relX/$(this).parent().width()));
				  		trimUpdate();
				  		
  						simPlayer.$self.prop('currentTime', trimStart);
					}
				});
				simPlayer.$trimHandleStop.draggable({containment: 'parent',
					drag: function(e, ui) {
						e.stopPropagation();
						
						if(ui.position.left <= simPlayer.$trimHandleStart.position().left)
							ui.position.left = simPlayer.$trimHandleStart.position().left;
				
						simPlayer.$trimMaskStop.css('width',$(this).parent().width() - Math.ceil($(this).position().left) - 25);	
					},
					stop: function (e,ui) {
						
						if ($(e.target).attr('class') === ".sp-Scrub") {
				           e.stopPropagation();
				            return false;
				        } 
						
						if(ui.position.left <= simPlayer.$trimHandleStart.position().left)
							ui.position.left = simPlayer.$trimHandleStart.position().left;
				
						simPlayer.$trimMaskStop.css('width',$(this).parent().width() - Math.ceil($(this).position().left) - 25);	

						var parentOffset = $(this).parent().offset(); 
					  	var maskleft = simPlayer.$trimMaskStop.offset().left;
					  	
					  	var relX = maskleft - parentOffset.left;
					  	//var relX = e.pageX - parentOffset.left - 4 + 25;
					  	
				  	   	trimStop = (videoDuration * (relX/$(this).parent().width()));
				  	    
				  	    if(trimStop > videoDuration)
				  	    	trimStop = videoDuration;
				  	    
				  		trimUpdate();
				  		
  						simPlayer.$self.prop('currentTime', trimStop);
				  		
					}
				});
			}
			
			var loopPlayback = function () {
				if(simPlayer.$loopPlayback.hasClass('active'))
				{
					simPlayer.$loopPlayback.removeClass('active');
					loopMode = false;
				}
				else
				{
					simPlayer.$loopPlayback.addClass('active');
					loopMode = true;
				}
				simPlayer.$self.focus();

			}
			
			var frameForward = function () {
				if(options.drawingTools)
				{
					if(typeof(exitDraw) == 'function')
						exitDraw();
					if(typeof(simHideComment) == 'function')
						simHideComment();
				}
				
				var timeInc = 0.05;
				if(options.frameRate > 0)
				{
					timeInc = simPlayer.$self.prop('duration')/(simPlayer.$self.prop('duration') * options.frameRate);				
					timeInc.toFixed(9);
				}			
					
				playerCurrentTime = simPlayer.$self.prop('currentTime');
				playerCurrentTime += timeInc;	
				simPlayer.$self.prop('currentTime', playerCurrentTime);
				playerCurrentTime = simPlayer.$self.prop('currentTime');
				simPlayer.$self.focus();
			}
			
			var frameBackward = function() {
				if(options.drawingTools)
				{
					if(typeof(exitDraw) == 'function')
						exitDraw();
					if(typeof(simHideComment) == 'function')
						simHideComment();
				}
				
				var timeInc = 0.05;
				if(options.frameRate > 0)
				{
					timeInc = simPlayer.$self.prop('duration')/(simPlayer.$self.prop('duration') * options.frameRate);				
					timeInc.toFixed(9);
				}	
				playerCurrentTime = simPlayer.$self.prop('currentTime');
				playerCurrentTime -= timeInc;	
				simPlayer.$self.prop('currentTime', playerCurrentTime);
				simPlayer.$self.focus();
			}
			
			var initAnalytics = function () {
							
				$(window).bind('beforeunload', function() {
					// Do not write a partial if the spot is not playing, as we have already logged a partial view.
					if(firstUnloadCall && !simPlayer.$self.prop('paused'))
					{
						if(asyncVal != null)
							asyncVal = false;
						
						analyticsStopTime = simPlayer.$self.prop('currentTime');
						options.trackPartialCallback(videoDuration, analyticsStopTime, analyticsStartTime, analyticsStopTime);
			  			firstUnloadCall = false;
			  			
					}
				}); 
			
				// for safari on an iPad.
				$(window).bind('pagehide', function() {
					// Do not write a partial if the spot is not playing, as we have already logged a partial view.
					if(firstUnloadCall && !simPlayer.$self.prop('paused'))
					{
						if(asyncVal != null)
							asyncVal = false;

						analyticsStopTime = simPlayer.$self.prop('currentTime');
						options.trackPartialCallback(videoDuration, analyticsStopTime, analyticsStartTime, analyticsStopTime);
			  			firstUnloadCall = false;
					}
				});
			}
			
			// Error handling
			var handleErrors = function(e) {
				var errorObj = simPlayer.$self.prop('error');
				  			  
				// Special case for embedded reels  
				if(errorObj.code == 4 && $('.embedReelThumbs').length > 0)
				{
					var srcFile = simPlayer.$self.prop('src'); 
					var srcParts = srcFile.split("/");
					simPlayer.$self.prop('src','/assets/videos/' + srcParts[4]);
				} 
				else if(errorObj.code == 2 || errorObj.code == 4)
				{
					location.reload();
				}  
				else
				{  
					simPlayer.$errorInfo.html('Media Error: ' + errorObj.code + '<br/>' + errorObj.message);	
					simPlayer.$errorInfo.fadeIn(function() {$(this).click(function() { $(this).hide()})});
				}
			}	
			
			// Airplay handling
			var initAirplay = function () {	
				if (window.WebKitPlaybackTargetAvailabilityEvent) {
				    simPlayer.$self.bind('webkitplaybacktargetavailabilitychanged', function(event) {					    
				        switch (event.originalEvent.availability) {
				            case "available":
				                simPlayer.$airPlayBtn.css('display','');
				                break;
				            default:
				               simPlayer.$airPlayBtn.hide();
				               break;
				        }
				        simPlayer.$airPlayBtn.on('click', function() {
				            simPlayer.$self[0].webkitShowPlaybackTargetPicker();
				        });
				    });
				}
			}
			
			// Time format selector
			var initTimeFormat = function() {
				simPlayer.$timeFormat.click(function() {
					simPlayer.$timeFormatSelect.show();

				});	
				simPlayer.$timeFormatSelect.mouseleave(function() {	
					simPlayer.$timeFormatSelect.hide();
				});
				
				simPlayer.$timeFormatOption.click(function(e) {
					e.preventDefault();
				
					timeDisplay = $(this).attr('value');
					simPlayer.$totalTime.html(timeFormat(simPlayer.$self.prop('duration')));
					localStorage.setItem('simiantimedisplay',timeDisplay);
					showProgress();
					simPlayer.$self.focus();
					simPlayer.$timeFormatSelect.hide();

				});
			}
			
			// Grid Overlay Selector
			var initGridOverlay = function() {
				simPlayer.$gridMenu.click(function() {
					simPlayer.$gridMenuSelect.show();

				});	
				simPlayer.$gridMenuSelect.mouseleave(function() {	
					simPlayer.$gridMenuSelect.hide();
				});
				
				simPlayer.$gridMenuOption.click(function(e) {
					e.preventDefault();
					gridOption = $(this).attr('value');
					
					var vWidth = simPlayer.$self.width();
					var vHeight = simPlayer.$self.height();
					
					var rawPlayer = document.getElementById("simianPlayer");
					var rawWidth = rawPlayer.videoWidth;
					var rawHeight = rawPlayer.videoHeight;
						
					var leftOffset = 0;	
					var rawAspect = (rawWidth/rawHeight);
					var playerWidth = vWidth;
					vWidth = (vHeight * rawAspect);
					
					if(vWidth != rawWidth)
					{
						vWidth += 2;
					}
					
					leftOffset = ((playerWidth - vWidth)/2);
					
					if(gridOption == 'off')
					{
						$('#aspectMaskOverlay').remove();
						simPlayer.$gridMenu.find('span').html('ASPECT MASK : OFF');
					}	
					else
					{	
						// Remove any old mask
						$('#aspectMaskOverlay').remove();

						// Determine current size:
						var currentAspect = (vWidth/vHeight).toFixed(3);
						
						var newAspect = gridOption.replace
						/*
						1.778 = 16:9
						1.333 = 4:3
						2.39 = 2.39:1
						2.333 = 21:0
						1.85 = 1.85:1
						1.5 = 3:2
						1 = 1:1
						
						newHeight = ( newWidth / aspectRatio ) 
						newWidth = ( newHeight * aspectRatio ) 
						
						*/
						
						if(gridOption == '16:9')
						{
							if(vWidth > vHeight || vWidth == vHeight)
							{
								var nWidth = (vHeight)*(16/9);
								var sliceWidth = (vWidth - nWidth)/2;
								
								if(nWidth > vWidth)
								{
								var nHeight = (vWidth)/(16/9);
								var sliceWidth = (vHeight - nHeight)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-bottom:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;width:' + (vWidth*0.9) + 'px;height:' + ((vHeight - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;bottom:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;position:absolute;border-top:1px solid var(--color-accent);"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);									

								}
								else
								{
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:left;border-right:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;height:' + (vHeight*0.9) + 'px;width:' + ((vWidth - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:right;border-left:1px solid var(--color-accent);"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
								}
							}
							else
							{
								var nHeight = (vWidth)/(16/9);
								var sliceWidth = (vHeight - nHeight)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-bottom:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;width:' + (vWidth*0.9) + 'px;height:' + ((vHeight - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;bottom:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;position:absolute;border-top:1px solid var(--color-accent);"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);									

							}	
						}
						else if(gridOption == '4:3')
						{
							// Calculate Mask
							if(vWidth > vHeight && (rawWidth != rawHeight))
							{
								var nWidth = (vHeight)*(4/3);
								var sliceWidth = (vWidth - nWidth)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:left;border-right:1px solid var(--color-accent);"></div>';
								
								console.log('FOOOOOk');
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;height:' + (vHeight*0.9) + 'px;width:' + ((vWidth - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								
								
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:right;border-left:1px solid var(--color-accent);"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
							}
							else
							{
								var nHeight = (vWidth)/(4/3);
								var sliceWidth = (vHeight - nHeight)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-bottom:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;width:' + (vWidth*0.9) + 'px;height:' + ((vHeight - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;bottom:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;position:absolute;border-top:1px solid var(--color-accent);"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
							}	
						}
						else if(gridOption == '1:1')
						{
							// Calculate Mask
							if(vWidth > vHeight)
							{
								var nWidth = vHeight;
								var sliceWidth = (vWidth - nWidth)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:left;border-right:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;height:' + (vHeight*0.9) + 'px;width:' + ((vWidth - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:right;border-left:1px solid var(--color-accent);"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
							}
							else
							{
								var nHeight = vWidth;
								var sliceWidth = (vHeight - nHeight)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-bottom:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;width:' + (vWidth*0.9) + 'px;height:' + ((vHeight - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;bottom:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-top:1px solid var(--color-accent);position:absolute;"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
							}
						}
						else if(gridOption == '21:9')
						{
							//if(vWidth > vHeight || vWidth == vHeight)
						//	if( vHeight > vWidth || vWidth == vHeight)
							{
								var nHeight = (vWidth)/(21/9);
								var sliceWidth = (vHeight - nHeight)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-bottom:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;width:' + (vWidth*0.9) + 'px;height:' + ((vHeight - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;bottom:0;left:0;height:' + sliceWidth + 'px;width:' + vWidth + 'px;border-top:1px solid var(--color-accent);position:absolute;"></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
							}/*
							else
							{
								var nWidth = (vHeight)*(21/9);
								var sliceWidth = (vWidth - nWidth)/2;
								
								// Create new dynamic overlay div
								var gol = '<div id="aspectMaskOverlay" style="width:' + vWidth + 'px;height:' + vHeight + 'px;position:absolute;z-index:555;top:0;left:' + leftOffset + 'px;">';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:left;border-right:1px solid var(--color-accent);"></div>';	
								gol += '<div style="position:absolute;right:0;left:0;top:0;bottom:0;margin:auto;background-color:transparent;height:' + (vHeight*0.9) + 'px;width:' + ((vWidth - (sliceWidth*2)) *0.9) + 'px;border:1px solid var(--color-accent);"></div>';
								gol += '<div style="background-color:#000;opacity:0.8;top:0;left:0;height:' + vHeight + 'px;width:' + sliceWidth + 'px;float:right;border-left:1px solid var(--color-accent);></div>';	
								gol += '</div>';
								$('.simian-player').prepend(gol);	
							}*/
						}
						simPlayer.$gridMenu.find('span').html('ASPECT MASK : ' + gridOption);
						simPlayer.$gridMenuSelect.hide();
					}											
				});
			}
			
			var playerInfo = function () {
				if(fullscreenMode)
					return false;				
				simPlayer.$self.trigger('pause');
				simPlayer.$playerInfo.fadeIn(function() {$(this).click(function() { $(this).hide()})});
				return false;
			}

			// Keyboard Controls
			/*var keyboardDisables = function (e) {
				if(e.keyCode == 32) {
					e.preventDefault();
  				}
			}*/


			var keyboardControls = function (e) {
				
				if($(document.activeElement).attr('id') == 'simianPlayer')
				{
					if (e.keyCode == 32 && reversePlay) 
					{
						reversePlay = false;
				  	}
				  	else if (e.keyCode == 32) 
					{
						playMedia();
				  	}
				  	else if(e.keyCode == 74 && simPlayer.$self.prop('paused')) // J
				  	{
					  	if(options.frameControls)
					  	{
				  			reversePlay = true;
				  			playInReverse();
				  		}
				  	}
				  	else if(e.keyCode == 75 && !simPlayer.$self.prop('paused')) // K
				  	{
 				  		if(options.frameControls)
	 				  	{
		 				  	reversePlay = false;
					  	  	playMedia(); 	
					  	}
				  	}
				  	else if(e.keyCode == 75 && reversePlay) // K
				  	{
 				  		if(options.frameControls)
	 				  	{
		 				  	reversePlay = false;
					  	}
				  	}
				  	else if(e.keyCode == 76 && simPlayer.$self.prop('paused')) // L
				  	{
 				  		if(options.frameControls)
						{
							reversePlay = false;
					  	  	playMedia();	
					  	}
				  	}
				  	else if(e.keyCode == 37 && !e.shiftKey) { // left
 				  		
					  	if(options.frameControls)
					  	{
					  		reversePlay = false;
					  		$('.sp-FrameBack').trigger('click');
					  	}
		  			}
		  			else if(e.keyCode == 39  && !e.shiftKey) { // right
 				  		
			  			if(options.frameControls)
				  		{
					  		reversePlay = false;
					  		$('.sp-FrameForward').trigger('click');
					  	}
					}
			    }
			}
			
			var playInReverse = function() {
				playerCurrentTime = simPlayer.$self.prop('currentTime');
				playerCurrentTime += -(1/options.frameRate);
				simPlayer.$self.prop('currentTime', playerCurrentTime);
				if(reversePlay)
				{
					setTimeout(playInReverse, 1000 / options.frameRate);
				}
			}
			
			var resizePlayer = function() {
				var playerWidth = $('#videoArea').width();
				
				if(options.gridOverlay)
				{
					//if(playerWidth < 750)
					//	simPlayer.$gridMenu.hide();
					//else
						simPlayer.$gridMenu.show();
				
					simPlayer.$gridMenu.find('span').html('ASPECT MASK : OFF');
					$('#aspectMaskOverlay').remove();
				}
			}
			
			// Initialize the player
			var init = function() {
				// attach playback handlers
				simPlayer.$playBtn.bind( (is_touch_device) ? 'touchstart' : 'click', playMedia);

				if (document.createElement('video').canPlayType) { // && !options.drawingTools) {
					simPlayer.$self.bind((is_touch_device) ? 'touchstart' : 'click', playMedia);
				}

				simPlayer.$self.bind('canplay', initVideo);
				simPlayer.$self.bind('play', startPlayback);
				simPlayer.$self.bind('pause', stopPlayback);
				simPlayer.$self.bind('ended', endPlayback);
				simPlayer.$self.bind('timeupdate', showProgress);
				simPlayer.$self.bind('progress', showBuffer);
				simPlayer.$self.bind('error', handleErrors);
				simPlayer.$self.bind('contextmenu', playerInfo);
				
				$(window).bind('resize', resizePlayer);
				$(window).bind('keyup', keyboardControls);
				//$(window).bind('keydown', keyboardDisables);
				
				simPlayer.$self.bind('fullscreenchange', exitFullscreen);
				simPlayer.$self.bind('webkitfullscreenchange', exitFullscreen);
				simPlayer.$self.bind('mozfullscreenchange', exitFullscreen);
				simPlayer.$self.bind('MSFullscreenChange', exitFullscreen);

				// bind Fullscreen Button
				if(options.fullScreen || options.fullScreenTop)
					simPlayer.$fullscreenBtn.click(goFullscreen);
				
				// bind Loop Video Button		
				if(options.loopPlayback)
					simPlayer.$loopPlayback.click(loopPlayback);
				
				// bind Frame navigation	
				if(options.frameControls)
				{
					simPlayer.$frameForward.click(frameForward);	
					simPlayer.$frameBack.click(frameBackward);	
				}
				// initialize volume controls
				initVolume();
				initSeek();
				
				if(options.timeFormatControl)
					initTimeFormat();
					
				if(options.gridOverlay)
					initGridOverlay();	
				
				if(options.trackAnalytics)
					initAnalytics();

				if(options.airplay)
					initAirplay();	
					
				// Wait til metadata loaded		
				simPlayer.$self.bind('loadedmetadata', function() {
				var t = window.setInterval(function() {
					if (simPlayer.$self.prop('readyState') > 0) {
							loadedMetadata = true;
							updateSeek();
							clearInterval(t);
							if(options.trimPlayback)
								initTrimPlayback();
				
				
						if(autoplay){
					$('.videoPlayOverlay').remove();
					 simPlayer.$self.trigger('play');
				}	
								
						}
					}, 500);
				});

				// trigger update seek manualy for the first time, for iOS support
				//showProgress();

				// remove the native controls
				simPlayer.$self.prop('controls', false);

				// if an autoplay attribute was set, play the video
				$('.videoPlayOverlay').click(function() {
					playMedia();
					
				});
				
				// Just in case
				resizePlayer();
				
			}();
		};

		// Public Methods 


		// Get the current time
		$.fn.simianVideoPlayer.isPlaying = function() {
			if(simPlayer.$self.prop('paused'))
				return false;
			else
				return true;
		}

		// Get the current time
		$.fn.simianVideoPlayer.updateTimeDisplay = function() {
		}

		// Get the current time
		$.fn.simianVideoPlayer.pauseVideo = function() {
			return simPlayer.$self.trigger('pause');
		}
		
		// Get the current time
		$.fn.simianVideoPlayer.getCurrentTime = function() {
			return simPlayer.$self.prop('currentTime');
		}
		
		$.fn.simianVideoPlayer.formatPlayerTime = function(secs) {
			return simPlayer.$formatTheTime(secs);
		}
		

		// Get the current time
		$.fn.simianVideoPlayer.getDuration = function() {
			
			var done = false;
			var count = 0;

			while(simPlayer.$self.prop('readyState') <= 0 && !done) 
			{
				// Wait til we ready
				count++;
				
				if(count >= 100000)
					done = true;
			}
			
			if(simPlayer.$self.prop('duration') > 0)
				return simPlayer.$self.prop('duration');
			else
				return 0;
		}
		
		// Get the x coordinate of the scrub head
		$.fn.simianVideoPlayer.getCurrentX = function() {
			return $('.sp-Scrub-slider .ui-slider-range').width();
		}
		
		// Get the width of the scrub bar
		$.fn.simianVideoPlayer.getScrubWidth = function() {
			return simPlayer.$seek.width();
		}
		
		// Get the x coordinate of the start of the scrub bar
		$.fn.simianVideoPlayer.getStartX = function() {
			return simPlayer.$seek.position().left;
		}		

		$.fn.simianVideoPlayer.getStopX = function() {
			return simPlayer.$seek.width() - simPlayer.$seek.position().left;
		}	

		// Jump to specified timestamp
		$.fn.simianVideoPlayer.setCurrentTime = function(timeStamp) {
			simPlayer.$self.trigger('pause');
			simPlayer.$self.prop('currentTime', timeStamp);
		}

		// Get time based on pixel position
		$.fn.simianVideoPlayer.xcoordToTime = function(pixel) {
			 return ((simPlayer.$self.prop('duration') * pixel)/ simPlayer.$seek.width());
		}
		
		$.fn.simianVideoPlayer.setRange = function(rStart, rStop) {
			if(rStop > rStart)
			{
				rangeStart = rStart;
				rangeStop = rStop;
				simPlayer.$loopPlayback.addClass('active');
				loopMode = true;
			}
		}
		
		$.fn.simianVideoPlayer.clearRange = function() {
			rangeStart = 0;
			rangeStop = 0;
			simPlayer.$loopPlayback.removeClass('active');
			loopMode = false;
		}		

		// Load new video into existing player
		$.fn.simianVideoPlayer.loadVideo = function(lVideoFile, lAutolpay, lTrimStart, lTrimStop, prevID = '') {
						
			// Capture last stats if we moved away from a playing video
			if(options.trackAnalytics && prevID !== '' && !endVideoCallBack)
			{				
				analyticsStopTime = simPlayer.$self.prop('currentTime');
				var elapsed = analyticsStopTime - analyticsStartTime;
				elapsed = (elapsed / simPlayer.$self.prop('duration')) * 100;

				if(elapsed > 99)
				{
					options.trackViewCallback('full', 100,analyticsStartTime, analyticsStopTime, prevID);	
				}
				else
					options.trackViewCallback('partial', elapsed,analyticsStartTime, analyticsStopTime, prevID);	
			}
						
			// Re-initialize
			seeking = false;
			wasPlaying = false;
			videoFile = lVideoFile;
			autoplay = lAutolpay;
			
			endVideoCallBack = false;

			// Analytics Variables
			playerCurrentTime = 0;
			analyticsStartTime = 0;
			analyticsStopTime = 0;
		  	firstUnloadCall = true;
				
			// Trim Variables
			trimStart = lTrimStart;
			trimStop = lTrimStop;
			trimPlaybackStopped = false;
			
			// Clear old poster frame
			simPlayer.$self.prop('poster','');			
			// Set new video src
			simPlayer.$self.prop('src',lVideoFile);
			
			// Wait til metadata loaded	for slow ass browsers like Safari	
			simPlayer.$self.bind('loadedmetadata', function() {
			
			// Do this quick - for Chrome & Firefox cause they slow	
			if(options.trimPlayback && (trimStart != 0 && trimStop != 0))
			{			
				simPlayer.$self.prop('currentTime', trimStart);
				analyticsStartTime = trimStart;
			}	
				
			var t = window.setInterval(function() {
				if (simPlayer.$self.prop('readyState') > 0) {
					loadedMetadata = true;
					clearInterval(t);
					simPlayer.$self.trigger('play');
				}
				}, 500);
			});
		}
	
		// Destory the player & cleanup
		$.fn.simianVideoPlayer.destroyPlayer = function() {

			$( "video#simianPlayer" ).each(function( index ) {
				$(this).trigger('pause');
				$(this).attr('src','');
				$(this).remove();
			});
							
			if(!simPlayer.$self.prop('paused'))
			{
				simPlayer.$self.trigger('pause');
			}
			
			$(window).unbind('keyup');
			
			simPlayer.$self.remove();
		}
		
		return this.each(simianPlayer);		
	};
})(jQuery);

/*
abort				Fires when the loading of an audio/video is aborted
canplay				Fires when the browser can start playing the audio/video
canplaythrough		Fires when the browser can play through the audio/video without stopping for buffering
durationchange		Fires when the duration of the audio/video is changed
emptied				Fires when the current playlist is empty
ended				Fires when the current playlist is ended
error				Fires when an error occurred during the loading of an audio/video
loadeddata			Fires when the browser has loaded the current frame of the audio/video
loadedmetadata		Fires when the browser has loaded meta data for the audio/video
loadstart			Fires when the browser starts looking for the audio/video
pause				Fires when the audio/video has been paused
play				Fires when the audio/video has been started or is no longer paused
playing				Fires when the audio/video is playing after having been paused or stopped for buffering
progress			Fires when the browser is downloading the audio/video
ratechange			Fires when the playing speed of the audio/video is changed
seeked				Fires when the user is finished moving/skipping to a new position in the audio/video
seeking				Fires when the user starts moving/skipping to a new position in the audio/video
stalled				Fires when the browser is trying to get media data, but data is not available
suspend				Fires when the browser is intentionally not getting media data
timeupdate			Fires when the current playback position has changed
volumechange		Fires when the volume has been changed
waiting				Fires when the video stops because it needs to buffer the next frame
*/